# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-10-23 12:29
from __future__ import unicode_literals

from django.db import migrations
from django.core.management.sql import emit_post_migrate_signal


class Migration(migrations.Migration):
    def update_permissions_for_group(apps, schema_editor):
        '''
        Update permissions for some users.

        Give bulk-delete permissions to moderators.
        Give edit permission to moderators and editors in order
        to display 'Main' page in the explorer.
        '''
        db_alias = schema_editor.connection.alias
        try:
            # Django 1.9
            emit_post_migrate_signal(2, False, db_alias)
        except TypeError:
            # Django < 1.9
            try:
                # Django 1.8
                emit_post_migrate_signal(2, False, 'default', db_alias)
            except TypeError:  # Django < 1.8
                emit_post_migrate_signal([], 2, False, 'default', db_alias)

        Group = apps.get_model('auth.Group')
        Permission = apps.get_model('auth.Permission')
        GroupPagePermission = apps.get_model('wagtailcore.GroupPagePermission')
        SectionIndexPage = apps.get_model('core.SectionIndexPage')
        MainPage = apps.get_model('core.Main')

        moderator_group = Group.objects.filter(name='Moderators').first()
        editor_group = Group.objects.filter(name='Editors').first()

        if moderator_group:
            sections = SectionIndexPage.objects.first()
            GroupPagePermission.objects.get_or_create(
                group_id=moderator_group.id,
                page_id=sections.id,
                permission_type='bulk_delete'
            )

            main = MainPage.objects.first()
            GroupPagePermission.objects.get_or_create(
                group_id=moderator_group.id,
                page_id=main.id,
                permission_type='edit'
            )

        if editor_group:
            main = MainPage.objects.first()
            GroupPagePermission.objects.get_or_create(
                group_id=editor_group.id,
                page_id=main.id,
                permission_type='edit'
            )


    dependencies = [
        ('core', '0051_remove_user_permission_for_moderator_group'),
        ('contenttypes', '__latest__'),
        ('wagtailcore', '__latest__'),
        ('wagtailadmin', '__latest__'),
        ('wagtailusers', '__latest__'),
        ('sites', '__latest__'),
        ('auth', '__latest__'),
    ]

    operations = [
        migrations.RunPython(
            update_permissions_for_group),
    ]



from __future__ import absolute_import, unicode_literals

from django.core.management.base import BaseCommand
from molo.core.page_utils import move_page_links_to_recommended_articles


class Command(BaseCommand):
    help = ("Iterates through all articles in the database and moves any "
            "articles at the end of a StreamField to Recommended Articles.")

    def handle(self, **options):
        move_page_links_to_recommended_articles()



# -*- coding: UTF-8 -*-
import csv
import pytz
from datetime import datetime
from django.core.management import call_command
from django.test import TestCase
from django.utils.six import StringIO
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.models import (
    Main, Languages, ArticlePage, Tag, ArticlePageTags, SiteLanguageRelation)


class ManagementCommandsTest(TestCase, MoloTestCaseMixin):
    def setUp(self):
        self.mk_main()
        self.main = Main.objects.all().first()
        self.language_setting = Languages.objects.create(
            site_id=self.main.get_site().pk)
        self.english = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(self.main.get_site()),
            locale='en',
            is_active=True)
        self.yourmind = self.mk_section(
            self.section_index, title='Your mind')
        self.article = ArticlePage(title='Test Article', slug='test-article')
        self.yourmind.add_child(instance=self.article)
        self.article.save_revision().publish()

    def test_add_feature_in_latest_date_to_article(self):
        self.assertEquals(self.article.featured_in_latest_start_date, None)
        now = datetime.now()
        now = pytz.utc.localize(now)

        # create the csv file with the article slug and date
        data = {
            self.article.slug: now
        }
        with open('dict.csv', 'wb') as f:
            writer = csv.writer(f)
            writer.writerow(['slug', 'date'])
            for key, value in data.items():
                writer.writerow([key, value])
        # call the command
        out = StringIO()
        call_command(
            'set_promotional_dates_from_csv', 'dict.csv', std_out=out)
        self.assertEquals(
            ArticlePage.objects.last().featured_in_latest_start_date, now)

    def test_add_tag_to_article(self):
        self.assertEquals(ArticlePageTags.objects.count(), 0)
        # create the tag data
        tag = Tag(title='over 18', slug='over-18')
        self.tag_index.add_child(instance=tag)
        tag.save_revision().publish()
        tag_data = {
            self.article.slug: tag.title
        }

        # create csv
        with open('tag_dict.csv', 'wb') as f:
            writer = csv.writer(f)
            writer.writerow(['slug', 'date'])
            for key, value in tag_data.items():
                writer.writerow([key, value])

        # call command to link tag to article
        out = StringIO()
        call_command(
            'add_tag_to_article', 'tag_dict.csv', 'en',
            std_out=out)

        # it should create a Article Relationship with the Tag
        self.assertEquals(ArticlePageTags.objects.count(), 1)
        self.assertEquals(
            ArticlePageTags.objects.first().tag.title, tag.title)
        self.assertEquals(
            ArticlePageTags.objects.first().page, self.article)



from __future__ import absolute_import, unicode_literals

import csv
from babel import Locale
from django.core.management.base import BaseCommand
from molo.core.models import (
    Languages, Tag, ArticlePage, ArticlePageTags, Main, SectionIndexPage,
    TagIndexPage)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('csv_name', type=str)
        parser.add_argument('locale', type=str)

    def handle(self, *args, **options):
        csv_name = options.get('csv_name', None)
        locale = options.get('locale', None)
        mains = Main.objects.all()
        articles = {}
        with open(csv_name) as articles_tags:
            reader = csv.reader(articles_tags)
            if mains:
                for row in reader:
                    key = row[0]
                    articles[key] = row[1:]

        for main in mains:
            section_index = SectionIndexPage.objects.child_of(main).first()
            tag_index = TagIndexPage.objects.child_of(main).first()
            main_lang = Languages.for_site(main.get_site()).languages.filter(
                is_active=True, is_main_language=True).first()
            if section_index and tag_index and main_lang:
                if main_lang.locale == locale:
                    for article_slug in articles:
                        article = ArticlePage.objects.descendant_of(
                            section_index).filter(slug=article_slug).first()
                        if article:
                            for tag_title in articles.get(article_slug):
                                tag = Tag.objects.child_of(tag_index).filter(
                                    title=tag_title.strip()).first()
                                if tag:
                                    if not article.nav_tags.filter(
                                            tag__title=tag):
                                        article_page_tag = ArticlePageTags(
                                            page=article, tag=tag)
                                        article_page_tag.save()
                                    else:
                                        self.stdout.write(self.style.WARNING(
                                            'Tag "%s" has been already asigned'
                                            ' to "%s" in "%s"'
                                            % (tag, article, main)))
                                else:
                                    self.stdout.write(self.style.NOTICE(
                                        'Tag "%s" does not exist in "%s"'
                                        % (tag_title, main)))
                        else:
                            self.stdout.write(self.style.ERROR(
                                'Article "%s" does not exist in "%s"'
                                % (article_slug, main.get_site())))
                else:
                    self.stdout.write(self.style.NOTICE(
                        'Main language of "%s" is not "%s".'
                        ' The main language is "%s"'
                        % (main.get_site(), Locale(locale).english_name,
                            main_lang)))
            else:
                if not section_index:
                    self.stdout.write(self.style.NOTICE(
                        'Section Index Page does not exist in "%s"' % main))
                if not tag_index:
                    self.stdout.write(self.style.NOTICE(
                        'Tag Index Page does not exist in "%s"' % main))
                if not main_lang:
                    self.stdout.write(self.style.NOTICE(
                        'Main language does not exist in "%s"' % main))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def set_promote_start_date(apps, schema_editor):
    ArticlePage = apps.get_model("core", "ArticlePage")

    latest_articles = ArticlePage.objects.filter(featured_in_latest=True)
    homepage_articles = ArticlePage.objects.filter(featured_in_homepage=True)
    section_articles = ArticlePage.objects.filter(featured_in_section=True)

    for article in latest_articles:
      article.featured_in_latest_start_date = article.latest_revision_created_at
      article.save()
    for article in homepage_articles:
      article.featured_in_homepage_start_date = article.latest_revision_created_at
      article.save()
    for article in section_articles:
      article.featured_in_section_start_date = article.latest_revision_created_at
      article.save()

class Migration(migrations.Migration):

    dependencies = [
        ('core', '0049_add_social_media_links_to_site_settings'),
    ]

    operations = [
        migrations.RunPython(set_promote_start_date),
    ]



from django import template

from molo.core.models import MoloMedia


register = template.Library()


@register.inclusion_tag(
    'core/tags/media_listing_homepage.html',
    takes_context=True
)
def media_listing_homepage(context):
    return {
        'media': MoloMedia.objects.filter(feature_in_homepage=True),
        'request': context['request'],
    }



import os.path
import shutil
import pkg_resources

import click


def get_package(package_name):
    try:
        pkg = pkg_resources.get_distribution(package_name)
    except pkg_resources.DistributionNotFound:
        raise click.UsageError('%s is not installed.' % (package_name,))

    if not pkg_resources.resource_isdir(pkg.key, 'templates'):
        raise click.UsageError(
            '%s does not have a templates directory.' % (pkg.key,))
    return pkg


def get_template_dirs(package_name):
    return pkg_resources.resource_listdir(package_name, 'templates')


@click.command()
@click.argument('source_package')
@click.argument('target_package')
def unpack_templates(**kwargs):
    source_pkg = get_package(kwargs['source_package'])
    target_pkg = get_package(kwargs['target_package'])

    for directory in get_template_dirs(source_pkg.key):
        try:
            shutil.copytree(
                pkg_resources.resource_filename(
                    source_pkg.key, os.path.join('templates', directory)),
                pkg_resources.resource_filename(
                    target_pkg.key, os.path.join('templates', directory))
            )
        except OSError as e:
            raise click.ClickException(str(e))


@click.command()
@click.argument('app_name')
@click.argument('directory', default='')
@click.option('--author',
              help='The author\'s name',
              default='Praekelt Foundation')
@click.option('--author-email',
              help='The author\'s email address',
              default='dev@praekelt.com')
@click.option('--url',
              help='The URL for this application.')
@click.option('--license',
              help='The License to release the package under.',
              default='BSD')
@click.option('--require',
              help=('Define an extra package that needs to be included '
                    'in the requirements.txt'),
              multiple=True)
@click.option('--include',
              help=('Which libraries to include and add to INSTALLED_APPS. '
                    'Expected format (<python-module-name> <regex-for-urls>)'),
              nargs=2, multiple=True)
def scaffold(**kwargs):
    from molo.core.cookiecutter import cookiecutter
    from django.utils.crypto import get_random_string

    extra_context = kwargs.copy()
    # NOTE: we're going to create a directory for whatever value app_name
    #       is unless directory is specified.
    if not extra_context['directory']:
        extra_context['directory'] = extra_context['app_name']
    molo_package = pkg_resources.get_distribution('molo.core')
    extra_context['molo_version'] = molo_package.version

    # Create a random SECRET_KEY to put it in the main settings.
    chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    extra_context['secret_key'] = get_random_string(50, chars)

    cookiecutter(
        pkg_resources.resource_filename(
            'molo.core', os.path.join('cookiecutter', 'scaffold')),
        no_input=True,
        extra_context=extra_context)


@click.group()
def main():
    pass


main.add_command(scaffold, name='scaffold')
main.add_command(unpack_templates, name='unpack-templates')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def create_banner_index(apps, schema_editor):
    from molo.core.models import BannerIndexPage, Main
    main = Main.objects.all().first()

    if main:
        banner_index = BannerIndexPage(title='Banners', slug='banners')
        main.add_child(instance=banner_index)
        banner_index.save_revision().publish()


def create_section_index(apps, schema_editor):
    from molo.core.models import SectionIndexPage, Main
    main = Main.objects.all().first()

    if main:
        section_index = SectionIndexPage(title='Sections', slug='sections')
        main.add_child(instance=section_index)
        section_index.save_revision().publish()


def create_footer_index(apps, schema_editor):
    from molo.core.models import FooterIndexPage, Main
    main = Main.objects.all().first()

    if main:
        footer_index = FooterIndexPage(title='Footer pages', slug='footer-pages')
        main.add_child(instance=footer_index)
        footer_index.save_revision().publish()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0033_bannerindexpage_footerindexpage_sectionindexpage'),
    ]

    operations = [
        migrations.RunPython(create_banner_index),
        migrations.RunPython(create_section_index),
        migrations.RunPython(create_footer_index),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-12 18:55
from __future__ import unicode_literals

from django.db import migrations
import molo.core.blocks
import wagtail.wagtailcore.blocks
import wagtail.wagtailcore.fields
import wagtail.wagtailimages.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0036_auto_add_related_section_to_article'),
    ]

    operations = [
        migrations.AlterField(
            model_name='articlepage',
            name='body',
            field=wagtail.wagtailcore.fields.StreamField([(b'heading', wagtail.wagtailcore.blocks.CharBlock(classname=b'full title')), (b'paragraph', molo.core.blocks.MarkDownBlock()), (b'image', wagtail.wagtailimages.blocks.ImageChooserBlock()), (b'list', wagtail.wagtailcore.blocks.ListBlock(wagtail.wagtailcore.blocks.CharBlock(label=b'Item'))), (b'numbered_list', wagtail.wagtailcore.blocks.ListBlock(wagtail.wagtailcore.blocks.CharBlock(label=b'Item'))), (b'page', wagtail.wagtailcore.blocks.PageChooserBlock()), (b'media', molo.core.blocks.MultimediaBlock())], blank=True, null=True),
        ),
    ]



import logging

from django.core.exceptions import ObjectDoesNotExist
from wagtail.wagtailcore.models import Page
from wagtail.wagtailcore.blocks import StreamValue
from molo.core.models import (
    ArticlePage,
    ArticlePageRecommendedSections,
)


def create_recomended_articles(main_article, article_list):
    '''
    Creates recommended article objects from article_list
    and _prepends_ to existing recommended articles.
    '''

    # store existing recommended articles
    existing_recommended_articles = [
        ra.recommended_article.specific
        for ra in main_article.recommended_articles.all()]
    # delete existing recommended articles
    ArticlePageRecommendedSections.objects.filter(page=main_article).delete()

    for hyperlinked_article in article_list:
        ArticlePageRecommendedSections(
            page=main_article,
            recommended_article=hyperlinked_article).save()

    # re-create existing recommended articles
    for article in existing_recommended_articles:
        if article not in article_list:
            ArticlePageRecommendedSections(
                page=main_article,
                recommended_article=article).save()


def seperate_end_page_links(stream_data):
    '''
    Seperate out page blocks at the end of a StreamField.

    Accepts: List of streamfield blocks
    Returns: Tuple of 2 lists of blocks - (remaining body, final article)
    '''
    stream_data_copy = list(stream_data)
    end_page_links = []

    for block in stream_data_copy[::-1]:
        if block['type'] == 'page':
            end_page_links.insert(0, block)
            stream_data_copy.pop()
        else:
            break

    return (stream_data_copy, end_page_links)


def get_page_ids_from_page_blocks(block_list):
    '''
    Extract ids from list of page blocks

    Accepts: list of stream blocks all with type 'page'
    Returns: list of page ids
    '''
    id_list = []
    for block in block_list:
        id_list.append(block['value'])
    return id_list


def get_pages_from_id_list(id_list):
    '''
    Accepts: list of page ids
    Returns: list of specific page objects
    '''
    page_list = []
    for id_ in id_list:
        try:
            page_list.append(
                Page.objects.get(id=id_).specific)
        except ObjectDoesNotExist:
            logging.error(
                "Attempted to fetch non-existent"
                " page with id of {}".format(id_))
    return page_list


def move_page_links_to_recommended_articles():
    '''
    Derived from https://github.com/wagtail/wagtail/issues/2110
    '''
    articles = ArticlePage.objects.all()

    for article in articles:
        (remaining_blocks, linked_article_blocks) = seperate_end_page_links(
            article.body.stream_data)

        if linked_article_blocks:
            linked_article_ids = get_page_ids_from_page_blocks(
                linked_article_blocks)
            linked_articles = get_pages_from_id_list(linked_article_ids)
            create_recomended_articles(article, linked_articles)
            parent = article.get_parent().specific
            parent.enable_recommended_section = True
            parent.save()

        stream_block = article.body.stream_block
        article.body = StreamValue(stream_block,
                                   remaining_blocks,
                                   is_lazy=True)
        article.save()
